-- --------------------------------------------------------------------
-- Title      : Test vectors for testing fixed_sin.
-- Performs a simple SIN test.
-- Last Modified: $Date: 2004-09-02 15:08:00-04 $
-- RCS ID: $Id: test_fixed_sin.vhd,v 1.1 2004-09-02 15:08:00-04 l435385 Exp $
--
--  Created for VHDL-200X-ft, David Bishop (dbishop@vhdl.org) 
-- ---------------------------------------------------------------------------
entity test_fixed_sin is
  generic (
    quiet : BOOLEAN := false);          -- run quietly
end entity test_fixed_sin;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
use ieee.math_real.all;
use ieee.fixed_pkg.all;

architecture testbench of test_fixed_sin is
  -- copied from std_logic_textio

  -- purpose: reports an error
  procedure report_error (
    constant errmes   :    string;      -- error message
    actual            : in ufixed;        -- data from algorithm
    constant expected :    ufixed)  is    -- reference data
    variable L : line;
  begin  -- function report_error
    assert (actual = expected) report errmes & CR
      & "Actal:    " & to_string(actual) & CR
      & "Expected: " & to_string(expected) severity error;
    return;
  end procedure report_error;
  component fixed_sin is
    port (
      arg      : in  ufixed (1 downto -16);
      clk, rst : in  std_ulogic;
      res      : out ufixed (1 downto -11));
  end component fixed_sin;
  constant clock_period : time := 500 ns;  -- clock period
  signal stop_clock : boolean := false;  -- stop the clock
  signal    arg      :  ufixed (1 downto -16);
  signal clk, rst : std_ulogic;
  signal res      : ufixed (1 downto -11);
begin  -- architecture testbench
  DUT : fixed_sin
    port map (
      arg => arg,                       -- [in  unsigned (7 downto 0)]
      clk => clk,                       -- [in  std_ulogic]
      rst => rst,                       -- [in  std_ulogic]
      res => res);                      -- [out ufixed (6 downto -8)]
  -- purpose: clock driver
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  clkprc: process is

  begin  -- process clkprc
    if (not stop_clock) then
      clk <= '0';
      wait for clock_period/2.0;
      clk <= '1';
      wait for clock_period/2.0;
    else
      wait;
    end if;
  end process clkprc;

  -- purpose: reset driver
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  reset_proc: process is

  begin  -- process reset_proc

    rst <= '0';
    wait for clock_period * 2.0;
    rst <= '1';
    wait;
  end process reset_proc;  
  -- purpose: main test loop
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  tester: process is

  begin  -- process tester
    arg <= (others => '0');
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    arg <= to_ufixed (math_deg_to_rad, arg'high, arg'low);
    wait for clock_period;
    arg <= to_ufixed (math_pi/8.0, arg'high, arg'low);
    wait for clock_period;
    arg <= to_ufixed (math_pi_over_2, arg'high, arg'low);
    wait for clock_period;
    arg <= to_ufixed (math_pi_over_3, arg'high, arg'low);
    wait for clock_period;
    arg <= to_ufixed (3.0*math_deg_to_rad, arg'high, arg'low);
    wait for clock_period;
    arg <= to_ufixed (math_sqrt_2, arg'high, arg'low);
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait;
  end process tester;

  -- purpose: check the output of the tester
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  checktest: process is
  begin  -- process checktest
    wait for clock_period/2.0;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    waitloop: while (res = 0) loop
      wait for clock_period;
    end loop waitloop;
    report_error ( "sin 1 deg",
                   res,
                   to_ufixed (sin(math_deg_to_rad), res'high, res'low));
    wait for clock_period;
    report_error ( "sin PI/8",
                   res,
                   to_ufixed (sin(math_pi/8.0), res'high, res'low));
    wait for clock_period;
    report_error ( "sin PI/2",
                   res,
                   to_ufixed (sin(math_pi_over_2), res'high, res'low));
    wait for clock_period;
    report_error ( "sin PI/3",
                   res,
                   to_ufixed (sin(math_pi_over_3), res'high, res'low));
    wait for clock_period;
    report_error ( "sin 3 deg",
                   res,
                   to_ufixed (sin(3.0*math_deg_to_rad), res'high, res'low));
    wait for clock_period;
    report_error ( "sin (sqrt (2))",
                   res,
                   to_ufixed (sin(math_sqrt_2), res'high, res'low));
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    assert (false) report "Testing complete" severity note;
    stop_clock <= true;
    wait;    
  end process checktest;
end architecture testbench;
