-- --------------------------------------------------------------------
-- Title      : Test vectors for testing the "fphdl64_pkg_alg" package.
-- This testbench test the algorithemic functions.  This test is similar
-- to the "test_fpalg" funciton, but it uses the pareterized packages.
-- Last Modified: $Date: 2005-05-27 10:09:20-04 $
-- RCS ID: $Id: test_fpalg64.vhd,v 1.1 2005-05-27 10:09:20-04 l435385 Exp $
--
--  Created for VHDL-200X-ft, David Bishop (dbishop@vhdl.org) 
-- ---------------------------------------------------------------------------

entity test_fpalg64 is
  generic (
    quiet : BOOLEAN := false);          -- run quietly
end entity test_fpalg64;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
use ieee.std_logic_textio.all;
use ieee.math_real.all;
use ieee.fphdl64_pkg.all;
use ieee.fphdl_base_pkg.all;
use work.fphdl_alg_pkg.all;
architecture testbench of test_fpalg64 is

--  constant zerofp : fp := (others => '0');          -- zero
--  constant nanfp  : fp :=
--    (fp'high-1 downto -1 => '1', others => '0');   -- Signal NAN
--  constant qnanfp : fp :=
--    (fp'high-1 downto 0 => '1', fp'low => '1',
--     others                       => '0');          -- quiet NAN
--  constant pos_inffp : fp :=
--    (fp'high-1 downto 0 => '1', others => '0');    -- infinity
--  constant neg_inffp : fp :=
--    (fp'high downto 0 => '1', others => '0');      -- -infinity
--  constant neg_zerofp : fp :=
--    (fp'high => '1', others => '0');      -- negative zero
  
--  -- purpose: to_float wrapper
--  function to_fp (
--    arg : INTEGER)
--    return fp is
--  begin
--    return to_float (arg, fp'high, -fp'low);
--  end function to_fp;

--  -- purpose: to_float wrapper
--  function to_fp (
--    arg : real)
--    return fp is
--  begin
--    return to_float (arg, fp'high, -fp'low);
--  end function to_fp;

  -- purpose: reports an error
  procedure report_error (
    constant errmes   :    string;      -- error message
    actual            : in fp;        -- data from algorithm
    constant expected :    fp)  is    -- reference data
    variable L : line;
    variable fract1, fract2 : fp; -- fract
  begin  -- function report_error
    assert actual = expected
      report errmes & CR &
      "Actual   " & to_string (actual) & " ("
      & real'image(to_real(actual))& ") /= " & CR &
      "Expected " & to_string (expected) & " ("
      & real'image(to_real(expected))& ")"
      severity error;
    return;
  end procedure report_error;

  -- purpose: reports an error
  -- A little more forgiving, one bit off either way.
  procedure report_error63 (
    constant errmes   :    string;      -- error message
    actual            : in fp;        -- data from algorithm
    constant expected :    fp)  is    -- reference data
    variable L : line;
    variable fract1, fract2 : fp; -- fract
  begin  -- function report_error
    if actual /= expected then
      fract1 := nextafter (expected, pos_inffp);
      fract2 := nextafter (expected, neg_inffp);
      if (actual /= fract1) and (actual /= fract2) then
        report errmes & CR &
          "Actual   " & to_string (actual) & " ("
          & real'image(to_real(actual))& ") /= " & CR &
          "Expected " & to_string (expected) & " ("
          & real'image(to_real(expected))& ")"
          severity error;
      end if;
    end if;
    return;
  end procedure report_error63;

  -- purpose: reports an error
  -- Many of the algorithms in "math_real" are only good to > float32
  -- precision, so we resize then check.
  procedure report_error32 (
    constant errmes   :    string;      -- error message
    actual            : in fp;        -- data from algorithm
    constant expected :    fp)  is    -- reference data
    variable L : line;
    variable fract1, fract2 : float (fp'high downto -17); -- fract
  begin  -- function report_error
    fract1 := resize (float(actual), fract1'high, -fract1'low);
    fract2 := resize (float(expected), fract2'high, -fract2'low);
    assert fract1 = fract2
      report errmes & CR &
      "Actual   " & to_string (actual) & " ("
      & real'image(to_real(actual))& ") /= " & CR &
      "Expected " & to_string (expected) & " ("
      & real'image(to_real(expected))& ")"
      severity error;
    return;
  end procedure report_error32;
  -- purpose: reports an error
  -- Needed to compare strings with meta values in them.
  procedure report_error (
    constant errmes : in string;                -- error message
    actual          : in std_logic_vector;      -- data from algorithm
    expected        : in std_logic_vector)  is  -- reference data
    variable L : line;
  begin  -- function report_error
    if actual /= expected then
      report errmes severity error;
      L := new string'("Actual   ");
      write (L, actual);
      write (L, string'(" /= "));
      writeline (output, L);
      L := new string'("Expected ");
      write (L, expected);
      writeline (output, L);
    end if;
    return;
  end procedure report_error;
begin  -- architecture testbench

  -- purpose: Checks the conversion functions
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  Test_process : process is
    variable checknum   : fp;         -- Floating point output
    variable checknum2  : fp;         -- Floating point output
    variable checknum3  : fp;         -- Floating point output
    constant one : fp :=  
      (fp'high-2 downto 0 => '1', others => '0');  -- one
    variable checkint   : integer;      -- integer test
    variable checkreal  : real;         -- real test
    variable checkreal2 : real;         -- real test
    variable checkuns   : unsigned ( 15 downto 0 );  -- unsigned test
    variable checksign  : signed ( fp'high - 1 downto 0 ); -- check of signed
  begin  -- process Test_process
    checkreal := math_e;
    checknum := to_fp (checkreal);  -- E
    checknum2 := ln (checknum);
    report_error63 (" ln (e) error ",
                  checknum2,
                  to_fp(log(checkreal)));  -- 1.0 (damn close)
    checknum := one;  -- 1.0
    checknum2 := ln (checknum);
    report_error (" ln (1.0) error ",
                  checknum2,
                  zerofp);  -- 0.0
    checkreal := math_pi;
    checknum  := to_fp(checkreal);  -- PI
    checknum2 := ln (checknum);
    report_error63 (" ln (PI) error ",
                    checknum2,
                    to_fp (log (checkreal)));
    checkint := 36892987;
    checknum := to_fp(checkint);  -- 36892987
    checknum2 := ln (checknum);
    report_error (" ln (36892987) error ",
                  checknum2,
                  to_fp(log(real(checkint))));
    checknum := (-1 => '1', others => '0');  -- denormal
    checknum2 := ln (checknum);
    checkreal := to_real(checknum);
    report_error63 (" ln (denormal) error ",
                    checknum2,
                    to_fp(log (checkreal)));
    -- Check special cases
    checknum := zerofp;
    checknum2 := ln (checknum);
    report_error ("ln(0)", checknum2, neg_inffp);
    checknum := - zerofp;
    checknum2 := ln (checknum);
    report_error ("ln(-0)", checknum2, neg_inffp);
    checknum := pos_inffp;
    checknum2 := ln (checknum);
    report_error ("ln(+inf)", checknum2, pos_inffp);
    checknum := neg_inffp;
    checknum2 := ln (checknum);
    report_error ("ln(-inf)", to_slv(checknum2), to_slv(qnanfp));
    checknum := to_fp(-5);
    checknum2 := ln (checknum);
    report_error ("ln(-5)", to_slv(checknum2), to_slv(qnanfp));    
    checknum := (fp'high => '1', -1 => '1', others => '0');  -- -denormal
    checknum2 := ln (checknum);
    report_error ("ln(-denormal)", to_slv(checknum2), to_slv(qnanfp));
    checknum := nanfp;
    checknum2 := ln (checknum);
    report_error ("ln(nan)", to_slv(checknum2), to_slv(qnanfp));
    checknum := qnanfp;
    checknum2 := ln (checknum);
    report_error ("ln(qnan)", to_slv(checknum2), to_slv(qnanfp));
    -- sqrt
    checknum := to_fp (2);
    checknum2 := sqrt (checknum);
    report_error63 (" sqrt (2) error ",
                    checknum2,
                    "0011111111110110101000001001111001100110011111110011101111001101");
    -- NOTE:  constant MATH_SQRT_2 in math_real package is incorrect.
    checkreal := 0.5;
    checknum := to_fp (checkreal);
    checknum2 := sqrt (checknum);
    report_error63 (" sqrt (0.5) error ",
                  checknum2,
                  to_fp(sqrt(checkreal)));
    checkreal := MATH_PI;
    checknum := to_fp (checkreal);
    checknum2 := sqrt (checknum);
    report_error (" sqrt (PI) error ",
                  checknum2,
                  "0011111111111100010110111111100010010001101101001110111101101010");
    -- Note:  sqrt(PI) (constant MATH_SQRT_PI) in math_real incorrect.
    checknum := to_fp(25);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(5);            -- sqrt (25) = 5
    report_error ("Sqrt of 25 error", checknum2, checknum3);
    checknum := to_fp(81);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(9);            -- sqrt (81) = 9
    report_error ("Sqrt of 81 error", checknum2, checknum3);
    checknum := to_fp(36);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(6);            -- sqrt (36) = 6
    report_error ("Sqrt of 36 error", checknum2, checknum3);
    checknum := to_fp(49);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(7);            -- sqrt (49) = 7
    report_error ("Sqrt of 49 error", checknum2, checknum3);
    checknum := to_fp(0.04);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(0.2);          -- sqrt (1/25) = 1/5
    report_error ("Sqrt of 0.04 error", checknum2, checknum3);
    checknum := to_fp(625);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(25);           -- sqrt (625) = 25
    report_error ("Sqrt of 625 error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-1) := '1';               -- denormal number 2**-1023
    checknum3 := to_fp(2.0**(-1023));
    report_error ("2**-1023 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(sqrt (2.0) * (2.0**(-512)));
    report_error63 ("Sqrt 2**-1023 denormal error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-2) := '1';               -- denormal number 2**-1024
    checknum3 := to_fp(2.0**(-1024));
    report_error ("2**-1024 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(2.0**(-512));
    report_error ("Sqrt 2**-1024 denormal error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-3) := '1';               -- denormal number 2**-1025
    checknum3 := to_fp(2.0**(-1025));
    report_error63 ("2**-1025 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(sqrt (2.0) * (2.0**(-513)));
    report_error63 ("Sqrt 2**-1025 denormal error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-4) := '1';               -- denormal number 2**-1026
    checknum3 := to_fp(2.0**(-1026));
    report_error ("2**-1026 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(2.0**(-513));
    report_error ("Sqrt 2**-1026 denormal error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-5) := '1';               -- denormal number 2**-1027
    checknum3 := to_fp(2.0**(-1027));
    report_error ("2**-1027 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(sqrt (2.0) * 2.0**(-514));
    report_error63 ("Sqrt 2**-1027 denormal error", checknum2, checknum3);
    checknum := (others => '0');
    checknum (-6) := '1';               -- denormal number 2**-1028
    checknum3 := to_fp(2.0**(-1028));
    report_error ("2**-1028 computation", checknum, checknum3);
    checknum2 := sqrt (checknum);
    checknum3 := to_fp(2.0**(-514));
    report_error ("Sqrt 2**-1028 denormal error", checknum2, checknum3);
    -- Test special cases
    checknum := zerofp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (0)", checknum2, zerofp);
    checknum := - zerofp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (-0)", checknum2, - zerofp);
    checknum := pos_inffp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (+inf)", checknum2, pos_inffp);
    checknum := neg_inffp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (-inf)", to_slv(checknum2), to_slv(qnanfp));
    checknum := to_fp (-25);
    checknum2 := sqrt (checknum);
    report_error ("sqrt (-25)", to_slv(checknum2), to_slv(qnanfp));
    -- denormal number -2**-1028
    checknum := (fp'high => '1', -6 => '1', others => '0');
    checknum2 := sqrt (checknum);
    report_error ("sqrt (-denormal)", to_slv(checknum2), to_slv(qnanfp));
    checknum := nanfp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (nan)", to_slv(checknum2), to_slv(qnanfp));
    checknum := qnanfp;
    checknum2 := sqrt (checknum);
    report_error ("sqrt (qnan)", to_slv(checknum2), to_slv(qnanfp));
    -- trig test
    checkreal := MATH_PI/2.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error ("sin PI/2 (=90) miscompare",
                  checknum,
                  to_fp(sin(checkreal)));             -- 1.0
    checkreal := MATH_PI/4.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);
    report_error32 ("sin PI/4 (=45) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));             -- 0.7071
    checkreal := MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);
    report_error32 ("sin PI/180 (=1) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));             -- 0.017452406
    checkreal := MATH_PI/2.0 - MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin PI/2-PI/180 (=89) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := MATH_PI/2.0 + MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin PI/2+PI/180 (=91) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := MATH_PI/2.0 + MATH_PI/4.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin PI/2+PI/4 (=135) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := 2.0*MATH_PI - MATH_PI/2.0;  -- 270 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin 2*PI-PI/2 (=270) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));  -- -1
    checkreal := 2.0*MATH_PI - MATH_PI/1.8;  -- 260 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin 2*PI-PI/1.8 (=260) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := 2.0*MATH_PI - MATH_PI/4.0;  -- 325 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin 2*PI-PI/4 (=325) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := 2.0*MATH_PI;  -- 360 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error ("sin 2*PI (=360) miscompare",
                  checknum,
                  to_fp(sin(checkreal)));  -- 0
    checkreal := -MATH_PI/2.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error ("sin -PI/2 (=-90) miscompare",
                  checknum,
                  to_fp(sin(checkreal)));             -- 1.0
    checkreal := -MATH_PI/4.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);
    report_error32 ("sin -PI/4 (=-45) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));             -- 0.7071
    checkreal := -MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);
    report_error32 ("sin -PI/180 (=-1) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));             -- 0.017452406
    checkreal := -MATH_PI/2.0 + MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin -PI/2+PI/180 (=-89) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := -(MATH_PI/2.0 + MATH_PI/180.0);
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin -(PI/2+PI/180) (=-91) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := -(MATH_PI/2.0 + MATH_PI/4.0);
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin -(PI/2+PI/4) (=-135) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    
    checkreal := -2.0*MATH_PI + MATH_PI/2.0;  -- -270 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error ("sin -2*PI+PI/2 (=-270) miscompare",
                  checknum,
                  to_fp(sin(checkreal)));  -- 1
    checkreal := -(2.0*MATH_PI - MATH_PI/1.8);  -- -260 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin -(2*PI-PI/1.8) (=-260) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := -2.0*MATH_PI + MATH_PI/4.0;  -- -325 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error32 ("sin -2*PI-PI/4 (=-325) miscompare",
                    checknum,
                    to_fp(sin(checkreal)));
    checkreal := -2.0*MATH_PI;  -- 360 degrees
    checknum2 := to_fp (checkreal);
    checknum := sin (checknum2);  -- has rounding problems
    report_error ("sin -2*PI (=-360) miscompare",
                  checknum,
                  to_fp(sin(checkreal)));  -- 0
    -- special cases to_fp (MATH_PI_OVER_2)
    checknum := sin (zerofp);
    report_error ("sin 0", checknum, zerofp);
    checknum := sin (neg_zerofp);
    report_error ("sin -0", checknum, zerofp);
    checknum := sin (pos_inffp);
    report_error ("sin inf", to_slv(checknum), to_slv(qnanfp));
    checknum := sin (neg_inffp);
    report_error ("sin -inf", to_slv(checknum), to_slv(qnanfp));
    checknum := sin (nanfp);
    report_error ("sin nan", to_slv(checknum), to_slv(qnanfp));
    checknum := sin (qnanfp);
    report_error ("sin qnan", to_slv(checknum), to_slv(qnanfp));
    -- cos
    checkreal := MATH_PI;
    checknum2 := to_fp (checkreal);
    checknum := cos (checknum2);
    report_error ("cos PI (=180) miscompare",
                  checknum,
                  to_fp(cos(checkreal)));             -- -1.0
    checkreal := MATH_PI/4.0;
    checknum2 := to_fp (checkreal);
    checknum := cos (checknum2);
    report_error32 ("cos PI/4 (=45) miscompare",
                    checknum,
                    to_fp(cos(checkreal)));             -- 0.7071
    checkreal := MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := cos (checknum2);
    report_error32 ("cos PI/180 (=1) miscompare",
                    checknum,
                    to_fp(cos(checkreal)));             -- 0.017452406
    checkreal := MATH_PI/2.0 - MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := cos (checknum2);  -- has rounding problems
    report_error32 ("cos PI/2-PI/180 (=89) miscompare",
                    checknum,
                    to_fp(cos(checkreal)));
    -- special cases to_fp (MATH_PI_OVER_2)
    checknum := cos (zerofp);
    report_error ("cos 0", checknum, to_fp(1));
    checknum := cos (neg_zerofp);
    report_error ("cos -0", checknum, to_fp(1));
    checknum := cos (pos_inffp);
    report_error ("cos inf", to_slv(checknum), to_slv(qnanfp));
    checknum := cos (neg_inffp);
    report_error ("cos -inf", to_slv(checknum), to_slv(qnanfp));
    checknum := cos (nanfp);
    report_error ("cos nan", to_slv(checknum), to_slv(qnanfp));
    checknum := cos (qnanfp);
    report_error ("cos qnan", to_slv(checknum), to_slv(qnanfp));
    -- tan
    checkreal := MATH_PI/4.0;
    checknum2 := to_fp (checkreal);
    checknum := tan (checknum2);
    report_error32 ("tan PI/4 (=45) miscompare",
                    checknum,
                    to_fp(tan(checkreal)));             -- 0.7071
    checkreal := MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := tan (checknum2);
    report_error32 ("tan PI/180 (=1) miscompare",
                    checknum,
                    to_fp(tan(checkreal)));             -- 0.017452406
    checkreal := MATH_PI/2.0 - MATH_PI/180.0;
    checknum2 := to_fp (checkreal);
    checknum := tan (checknum2);  -- has rounding problems
    report_error32 ("tan PI/2-PI/180 (=89) miscompare",
                    checknum,
                    to_fp(tan(checkreal)));
    -- special cases
    checknum := tan (to_fp (MATH_PI));
    report_error ("tan PI", checknum, zerofp);
    if (not quiet) then
      report "Two tan(90) and tan(-90) almost infinity errors" severity note;
      checknum := tan (to_fp (MATH_PI_OVER_2));
      report_error ("tan PI/2", checknum, pos_inffp);
      checknum := tan (to_fp (MATH_PI + MATH_PI_OVER_2));
      report_error ("tan 270", checknum, neg_inffp);
    end if;
    checknum := tan (to_fp (2.0*MATH_PI));
    report_error ("tan 2*PI", checknum, zerofp);
    checknum := tan (zerofp);
    report_error ("tan 0", checknum, zerofp);
    checknum := tan (neg_zerofp);
    report_error ("tan -0", checknum, zerofp);
    checknum := tan (pos_inffp);
    report_error ("tan inf", to_slv(checknum), to_slv(qnanfp));
    checknum := tan (neg_inffp);
    report_error ("tan -inf", to_slv(checknum), to_slv(qnanfp));
    checknum := tan (nanfp);
    report_error ("tan nan", to_slv(checknum), to_slv(qnanfp));
    checknum := tan (qnanfp);
    report_error ("tan qnan", to_slv(checknum), to_slv(qnanfp));
    -- arcsin
    checknum3 := zerofp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(0)", checknum, zerofp);
    checknum3 := neg_zerofp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(-0)", checknum, zerofp);
    checknum3 := one;
    checknum := arcsin (checknum3);
    report_error ("arcsin(0)", checknum, to_fp (MATH_PI_OVER_2));
    checkreal := sin(MATH_PI/180.0);
    checknum3 := to_fp (checkreal);
    checknum := arcsin (checknum3);
    report_error32 ("arcsin(pi/180)", checknum, to_fp (MATH_PI/180.0));
    checkreal := sin(-MATH_PI/180.0);
    checknum3 := to_fp (checkreal);
    checknum := arcsin (checknum3);
    report_error32 ("arcsin(-pi/180)", checknum, to_fp (arcsin(checkreal)));
    -- wrong result?
--    checkreal := sin (MATH_PI_OVER_2 - (MATH_PI/180.0));
--    checknum3 := to_fp (checkreal);
--    checknum := arcsin (checknum3);
--    report_error ("arcsin(PI/2-pi/180)", checknum,
--                  to_fp (arcsin(checkreal)));
    checknum3 := one;
    checknum := arcsin (checknum3);
    report_error32 ("arcsin(1)", checknum, to_fp (MATH_PI_OVER_2));
    checknum3 := -one;
    checknum := arcsin (checknum3);
    report_error32 ("arcsin(-1)", checknum, to_fp (-MATH_PI_OVER_2));
    checknum3 := to_fp(1.1);
    checknum := arcsin (checknum3);
    report_error ("arcsin(1.1)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := to_fp(-1.1);
    checknum := arcsin (checknum3);
    report_error ("arcsin(-1.1)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := pos_inffp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(inf)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := neg_inffp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(-inf)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := nanfp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(nan)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := qnanfp;
    checknum := arcsin (checknum3);
    report_error ("arcsin(qinf)", to_slv(checknum), to_slv(qnanfp));
    -- arccos
    checknum3 := one;  -- 1.0
    checknum := arccos (checknum3);
    report_error32 ("arccos 1.0 miscompare",
                    checknum,
                    zerofp);             -- 0.0
    checknum3 := -one;  -- -1.0
    checknum := arccos (checknum3);
    report_error32 ("arccos -1.0 miscompare",
                    checknum,
                    zerofp);             -- 0.0
    checknum3 := zerofp;
    checknum := arccos (checknum3);
    report_error ("arccos(0)", checknum, to_fp (MATH_PI_OVER_2));
    checknum3 := neg_zerofp;
    checknum := arccos (checknum3);
    report_error ("arccos(-0)", checknum, to_fp (MATH_PI_OVER_2));
    checkreal := cos (MATH_PI_OVER_2 - (MATH_PI/180.0));
    checknum3 := to_fp (checkreal);
    checknum := arccos (checknum3);
    report_error32 ("arccos(PI/2-pi/180)", checknum,
                    to_fp (arccos(checkreal)));
    checkreal := cos(-MATH_PI_OVER_2+(MATH_PI/180.0));
    checknum3 := to_fp (checkreal);
    checknum := arccos (checknum3);
    report_error32 ("arccos(-PI/2+pi/180)", checknum,
                    to_fp (arccos(checkreal)));
    checknum3 := to_fp(1.1);
    checknum := arccos (checknum3);
    report_error ("arccos(1.1)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := to_fp(-1.1);
    checknum := arccos (checknum3);
    report_error ("arccos(-1.1)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := pos_inffp;
    checknum := arccos (checknum3);
    report_error ("arccos(inf)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := neg_inffp;
    checknum := arccos (checknum3);
    report_error ("arccos(-inf)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := nanfp;
    checknum := arccos (checknum3);
    report_error ("arccos(nan)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := qnanfp;
    checknum := arccos (checknum3);
    report_error ("arccos(qinf)", to_slv(checknum), to_slv(qnanfp));
    -- arctan
    checknum := arctan(zerofp);
    report_error ("arctan(0)", checknum, zerofp);
    checkreal := tan (MATH_PI/180.0);
    checknum := arctan (to_fp (checkreal));
    report_error32 ("arctan(pi/180)", checknum,
                    to_fp(arctan(checkreal)));
    checkreal := tan (MATH_PI/4.0);
    checknum := arctan (to_fp (checkreal));
    report_error32 ("arctan(45 deg)", checknum,
                  to_fp(arctan(checkreal)));
    checkreal := tan (-MATH_PI/180.0);
    checknum := arctan (to_fp (checkreal));
    report_error32 ("arctan(-pi/180)", checknum,
                    to_fp(arctan(checkreal)));
    checkreal := tan (-MATH_PI/4.0);
    checknum := arctan (to_fp (checkreal));
    report_error32 ("arctan(-45 deg)", checknum,
                  to_fp(arctan(checkreal)));
    checknum := arctan (neg_zerofp);
    report_error ("arctan(-0)", checknum, zerofp);
    checknum := arctan (pos_inffp);
    report_error ("arctan(+inf)", checknum, to_fp(MATH_PI_OVER_2));
    checknum := arctan (neg_inffp);
    report_error ("arctan(-inf)", checknum, to_fp(-MATH_PI_OVER_2));
    checknum := arctan (nanfp);
    report_error ("arctan(nan)", to_slv(checknum), to_slv(qnanfp));
    checknum := arctan (qnanfp);
    report_error ("arctan(qnan)", to_slv(checknum), to_slv(qnanfp));  
    -- **
    checknum2 := one;
    checknum3 := to_fp (2);
    checknum := checknum2 ** checknum3;
    report_error ("1**2 miscompare",
                  checknum,
                  one);
    checknum2 := to_fp (2);
    checknum3 := checknum2;
    checknum := checknum2 ** checknum3;
    report_error ("2**2 miscompare",
                  checknum,
                  to_fp(4));
    checknum2 := to_fp (3);
    checknum3 := to_fp (3);
    checknum := checknum2 ** checknum3;
    report_error63 ("3**3 miscompare",
                    checknum,
                    to_fp(27));
    checknum2 := to_fp (5);
    checknum3 := to_fp (5);
    checknum := checknum2 ** checknum3;
    report_error32 ("5**5 miscompare",
                    checknum,
                    to_fp(3125));

    -- cube root test
    checknum2 := to_fp (8);
    checknum := cbrt (checknum2);
    checknum3 := to_fp(2);
    report_error ("cbrt(8) = 2",
                  checknum, checknum3);
    checknum2 := to_fp (125);
    checknum := cbrt (checknum2);
    checknum3 := to_fp(5);
    report_error32 ("cbrt(125) = 5",
                    checknum, checknum3);
    checknum2 := to_fp (364);
    checknum := cbrt (checknum2);
    checknum3 := to_fp(cbrt(364.0));
    report_error63 ("cbrt(364) miscompare",
                    checknum, checknum3);
    checknum2 := to_fp(2197);         -- 13**3
    checknum := cbrt (checknum2);
    checknum3 := to_fp(13);
    report_error63 ("cbrt(2197) = 13", checknum, checknum3);
    checknum2 := to_fp(2.0**(-9));
    checknum := cbrt (checknum2);
    checknum3 := to_fp(2.0**(-3));
    report_error ("cbrt(2**-9) = 2**-3", checknum, checknum3);

    -- e**x test
    checknum := exp(one);
    checknum3 := to_fp(exp(1.0));
    report_error63 ("exp(1) error",
                    checknum, checknum3);
    checknum2 := to_fp (MATH_E);
    checknum := exp (checknum2);
    checknum3 := to_fp (exp (MATH_E));
    report_error32 ("exp(e) error",
                    checknum, checknum3);
    checknum2 := to_fp (MATH_PI);
    checknum := exp (checknum2);
    checknum3 := to_fp (exp (MATH_PI));
    report_error32 ("exp(PI) error",
                    checknum, checknum3);
--    checknum3 := fp (std_ulogic_vector(X"4023645FE268C05E"));
    -- log
    checkreal := 1000.0;
    checknum3 := to_fp(checkreal);
    checknum := log(checknum3, 10);
    checknum2 := to_fp(log10 (checkreal));
    report_error32 ("Log(1000.0, 10)", checknum, checknum2);
    checknum := log (checknum3, one);
    report_error ("Log(1000, 1)", checknum, zerofp);
    checknum := log (checknum3, checknum3);
    report_error ("Log(1000, 1000)", checknum, one);
    checknum := log (checknum3, zerofp);
    report_error ("Log(1000, 0)", checknum, neg_inffp);
    checknum := log (one, checknum3);
    report_error ("Log(1, 1000)", checknum, zerofp);
    checknum3 := to_fp (-1);
    checknum := log (checknum3, 10);
    report_error ("Log (-1, 10)", to_slv(checknum), to_slv(qnanfp));
    checknum3 := zerofp;
    checknum := log (checknum3, 10);
    report_error ("Log (0, 10)", checknum, neg_inffp);
    checknum3 := pos_inffp;
    checknum := log (checknum3, 10);
    report_error ("Log (inf, 10)", checknum, pos_inffp);    
    checkreal := 1000.0;
    checknum3 := to_fp(checkreal);
    checknum := log (zerofp, checknum3);
    report_error ("Log(0, 1000)", checknum, neg_inffp);
    assert (false) report "fphdl64_pkg algorithemic test complete"
      severity note;
    wait;
  end process Test_process;

end architecture testbench;
