-------------------------------------------------------------------------------
-- test routine for fp_sin
-------------------------------------------------------------------------------
entity test_fpsin is
  generic (
    quiet : BOOLEAN := false);          -- run quietly
end entity test_fpsin;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
use ieee.float_pkg.all;
use std.textio.all;
use ieee.std_logic_textio.all;
architecture testbench of test_fpsin is
  subtype  fp16 is float (6 downto -9);                           -- fp16 alias
  constant iter : INTEGER := 2;         -- interations
  procedure report_error (
    constant errmes   : in STRING;      -- error message
    actual            : in float;       -- data from algorithm
    constant expected : in float) is    -- reference data
    variable fract1, fract2 : float (fp16'high downto fp16'low);  -- fract
  begin  -- function report_error
    if (actual /= expected) then
      -- Create a spread of 2 bits off in either direction.
      fract2     := nextafter (expected, pos_inffp);
      fract2     := nextafter (fract2, pos_inffp);
      fract2     := nextafter (fract2, pos_inffp);
--      fract2     := nextafter (fract2, pos_inffp);
      fract1     := nextafter (expected, neg_inffp);
      fract1     := nextafter (fract1, neg_inffp);
      fract1     := nextafter (fract1, neg_inffp);
--      fract1     := nextafter (fract1, neg_inffp);
      if (actual <= fract2) and (actual >= fract1) then
--        report "Close match" severity warning;
      else
        report errmes & CR &
          "Actual   " & to_string (actual) & " ("
          & REAL'image(to_real(actual))& ") /= " & CR &
          "Expected " & to_string (expected) & " ("
          & REAL'image(to_real(expected))& ")"
          severity error;
      end if;
    else
--      report "Exact match" severity note;
    end if;
    return;
  end procedure report_error;

--  component quartus_sin is
--    port (
--      clk       : IN  std_logic;
--      rst       : IN  std_logic;
--      arg       : IN  std_logic_vector(6 DOWNTO 0);
--      arg_neg_5 : IN  std_logic;
--      arg_neg_7 : IN  std_logic;
--      arg_neg_6 : IN  std_logic;
--      arg_neg_4 : IN  std_logic;
--      arg_neg_8 : IN  std_logic;
--      arg_neg_9 : IN  std_logic;
--      arg_neg_1 : IN  std_logic;
--      arg_neg_3 : IN  std_logic;
--      arg_neg_2 : IN  std_logic;
--      res       : OUT std_logic_vector(6 DOWNTO 0);
--      res_neg_1 : OUT std_logic;
--      res_neg_2 : OUT std_logic;
--      res_neg_3 : OUT std_logic;
--      res_neg_4 : OUT std_logic;
--      res_neg_5 : OUT std_logic;
--      res_neg_6 : OUT std_logic;
--      res_neg_7 : OUT std_logic;
--      res_neg_8 : OUT std_logic;
--      res_neg_9 : OUT std_logic);
--  end component quartus_sin;
  component fp_sin is
    port (
      arg      : in  fp16;
      clk, rst : in  STD_ULOGIC;
      res      : out fp16);
  end component fp_sin;

  constant clock_period    : TIME    := 50 ns;  -- clock period
  signal   stop_clock      : BOOLEAN := false;  -- stop the clock
  signal   clk, rst_n, rst : STD_ULOGIC;        -- controls
  signal   insig, outsig   : fp16;              -- 16 bit fp
--  signal inslv, outslv : std_logic_vector (15 downto 0);  -- slv versions
  signal   inreal, outreal : REAL    := 0.0;    -- real versions
  signal   datapipe        : fp16;              -- delayed data
  signal   angr, sinr      : REAL    := 0.0;    -- real version of test
begin  -- architecture testbench
--  DUT: quartus_sin
--    port map (
--      clk       => clk,                 -- [IN  std_logic]
--      rst       => rst_n,                 -- [IN  std_logic]
--      arg       => inslv(15 downto 9),  -- [IN  std_logic_vector(6 DOWNTO 0)]
--      arg_neg_5 => insig(-5),           -- [IN  std_logic]
--      arg_neg_7 => insig(-7),           -- [IN  std_logic]
--      arg_neg_6 => insig(-6),           -- [IN  std_logic]
--      arg_neg_4 => insig(-4),           -- [IN  std_logic]
--      arg_neg_8 => insig(-8),           -- [IN  std_logic]
--      arg_neg_9 => insig(-9),           -- [IN  std_logic]
--      arg_neg_1 => insig(-1),           -- [IN  std_logic]
--      arg_neg_3 => insig(-3),           -- [IN  std_logic]
--      arg_neg_2 => insig(-2),           -- [IN  std_logic]
--      res       => outslv(15 downto 9), -- [OUT std_logic_vector(6 DOWNTO 0)]
--      res_neg_1 => outslv(8),           -- [OUT std_logic]
--      res_neg_2 => outslv(7),           -- [OUT std_logic]
--      res_neg_3 => outslv(6),           -- [OUT std_logic]
--      res_neg_4 => outslv(5),           -- [OUT std_logic]
--      res_neg_5 => outslv(4),           -- [OUT std_logic]
--      res_neg_6 => outslv(3),           -- [OUT std_logic]
--      res_neg_7 => outslv(2),           -- [OUT std_logic]
--      res_neg_8 => outslv(1),           -- [OUT std_logic]
--      res_neg_9 => outslv(0));          -- [OUT std_logic]
--  inslv <= to_slv(insig);
--  outsig <= to_fp(outslv);

  DUT : fp_sin
    port map (
      arg => insig,                     -- [in  fp16]
      clk => clk,                       -- [in  std_ulogic]
      rst => rst_n,                     -- [in  std_ulogic]
      res => outsig);                   -- [out fp16]

  -- purpose: sin wave
  -- type   : sequential
  -- inputs : clk, rst_n, insig
  -- outputs: outsig
--  sinproc: process (clk, rst_n) is

--  begin  -- process sinproc

--    if rst_n = '0' then                 -- asynchronous reset (active low)
--      outsig <= ( others => '0');
--    elsif rising_edge(clk) then         -- rising clock edge
--      outsig <= sin (insig);
----      outsig <= to_fp64 (sin (to_real(insig)));
--    end if;

--  end process sinproc;

  -- purpose: clock driver
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  clkprc : process is

  begin  -- process clkprc
    if (not stop_clock) then
      clk <= '0';
      wait for clock_period/2.0;
      clk <= '1';
      wait for clock_period/2.0;
    else
      wait;
    end if;
  end process clkprc;

  -- purpose: reset driver
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  reset_proc : process is

  begin  -- process reset_proc

    rst_n <= '0';
    wait for clock_period * 2.0;
    rst_n <= '1';
    wait;
  end process reset_proc;
  rst <= not rst_n;

  -- purpose: sends test vectors
  -- type   : combinational
  -- inputs : 
  -- outputs: inreal
  tester : process is
    variable inrealx : REAL;
    variable i       : INTEGER;         -- var
  begin  -- process tester
    inreal <= 0.0;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;

    inrealx := -0.25 * math_pi;
    inreal  <= inrealx;
    wait for clock_period;
    waitloopp : while (inreal < 0.25*math_pi) loop
      inrealx := inreal + 0.01;
      inreal  <= inrealx;
      wait for clock_period;
    end loop waitloopp;
    waitloopm : while (inreal > -0.25*math_pi) loop
      inrealx := inreal - 0.01;
      inreal  <= inrealx;
      wait for clock_period;
    end loop waitloopm;

    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait;
  end process tester;
  insig <= to_float(inreal, fp16'high, -fp16'low);

  outreal  <= to_real (outsig);
  datapipe <= to_float (angr, fp16'high, -fp16'low);

  -- purpose: check the output of the tester
  -- type   : combinational
  -- inputs : 
  -- outputs: 
  checktest : process is
    variable i                : INTEGER;                                -- var
    constant fxzero           : fp16 := zerofp (fp16'high, -fp16'low);  -- zero
    variable angreal, sinreal : REAL;
    variable checknum         : fp16;
  begin  -- process checktest
    wait for clock_period/2.0;
    wait for clock_period;
    wait for clock_period;
    resetloop : while (outsig = fxzero) loop
      wait for clock_period;
    end loop resetloop;

    angreal  := -0.25 * math_pi;
    sinreal  := sin (angreal);
    angr     <= angreal;
    sinr     <= sinreal;
    checknum := to_float (sinreal, checknum);
    report_error ("sin (" & REAL'image(angreal) & ") = "
                  & REAL'image(sinreal), outsig, checknum);
    wait for clock_period;
    waitloopp : while (angreal < 0.25*math_pi) loop
      angreal  := angreal + 0.01;
      sinreal  := sin (angreal);
      angr     <= angreal;
      sinr     <= sinreal;
      checknum := to_float (sinreal, checknum);
      report_error ("sin (" & REAL'image(angreal) & ") = "
                    & REAL'image(sinreal), outsig, checknum);
      wait for clock_period;
    end loop waitloopp;
    waitloopm : while (inreal > -0.25*math_pi) loop
      angreal  := angreal - 0.01;
      sinreal  := sin (angreal);
      angr     <= angreal;
      sinr     <= sinreal;
      checknum := to_float (sinreal, checknum);
      report_error ("sin (" & REAL'image(angreal) & ") = "
                    & REAL'image(sinreal), outsig, checknum);
      wait for clock_period;
    end loop waitloopm;

    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    wait for clock_period;
    report "16 bit floating point SIN test completed." severity note;
    stop_clock <= true;
    wait;
  end process checktest;
end architecture testbench;
